﻿using System.Collections;
using System.Linq;
using UnityEngine;

namespace AwasAutoJuice {
    internal class PussyDripAction : AwasFluidAction {

        public PussyDripAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) {
            afcloth.SetSubsurfaceColor(Color.white);
            afcloth.SetDiffuseOffset(0f);

            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetGravity(1.5f);
            // Don't set collision radius during action - it resets cloth
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            afcloth.SetFriction(actionConfig.friction);
        }

        protected override IEnumerator Start() {
            yield return WaitForEndOfFrame();
            afcloth.SetSimEnabled(true);
            afcloth.SyncSimValues();
            StartCoroutine(LerpAlphaToVisible());
            StartCoroutine(LerpJointStrength(0.5f));
            yield return LerpStiffness(0.5f);
            afcloth.SetDetached(true);
            afcloth.SetCompressionResistance(actionConfig.dripCompression);
            afcloth.SetDistanceScale(actionConfig.dripSize);
            StartCoroutine(LerpFriction(0.5f));
        }
    }

    internal class PussyJuicyAction : AwasFluidAction {

        public PussyJuicyAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) {
            afcloth.SetSubsurfaceColor(Color.white);
            afcloth.SetDiffuseOffset(0.0f);
            fadeInSeconds = 1.5f;
        }

        protected override IEnumerator Start() {
            yield return WaitForEndOfFrame();
            afcloth.SetSimEnabled(true);
            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetGravity(0f);
            afcloth.SetWeight(0f);
            afcloth.SetDrag(0.15f);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            afcloth.SetFriction(actionConfig.friction);
            afcloth.SyncSimValues();
            yield return LerpAlphaToVisible();
        }
    }

    internal class PussyFullOfCumAction : PussyJuicyAction {

        public PussyFullOfCumAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) {
            afcloth.SetSubsurfaceColor(new Color(0.97f, 0.95f, 0.95f, 0.95f));
            afcloth.SetDiffuseOffset(0.2f);
        }
    }

    internal class PussyWetAction : AwasFluidAction {

        public PussyWetAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) {
            fadeInSeconds = 1.5f;
        }

        protected override IEnumerator Start() {
            afcloth.SetDiffuseOffset(-0.2f);
            afcloth.SetSimEnabled(false);
            yield return WaitForEndOfFrame();
            yield return LerpAlphaToVisible();
        }
    }

    internal class PussyCumDripAction : PussyDripAction {

        public PussyCumDripAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) {
            afcloth.SetSubsurfaceColor(new Color(0.5f, 0.4f, 0.25f));
            afcloth.SetDiffuseOffset(0.2f);
        }
    }

    internal class MouthCumOnAction : AwasFluidAction {

        public MouthCumOnAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { 
            afcloth.SetDiffuseOffset(0.2f);
            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetFriction(actionConfig.friction);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
        }

        protected override IEnumerator Start() {
            yield return WaitForEndOfFrame();
            afcloth.SetSimEnabled(true);
            afcloth.SyncSimValues();
            yield return LerpAlphaToVisible();
        }
    }

    internal class MouthCumDripAction : AwasFluidAction {

        public MouthCumDripAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) {
            afcloth.SetSubsurfaceColor(Color.white);
            afcloth.SetDiffuseOffset(0.1f);
            
            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetGravity(1f);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            afcloth.SetFriction(actionConfig.friction);
        }

        protected override IEnumerator Start() {
            afcloth.SetStiffness(actionConfig.dripStiffness);
            afcloth.SetCompressionResistance(actionConfig.dripCompression);
            yield return WaitForEndOfFrame();
            afcloth.SetSimEnabled(true);
            afcloth.SyncSimValues();
            StartCoroutine(LerpAlphaToVisible());
            yield return new WaitForSeconds(0.2f);
            StartCoroutine(LerpFriction(2f));
            yield return LerpJointStrength(2f);
            afcloth.SetDetached(true);
            StartCoroutine(LerpDistanceScale(1f));
            afcloth.SetDistanceScale(actionConfig.dripSize); 
        }
    }

    internal class MouthCumStickyAction : AwasFluidAction {

        public MouthCumStickyAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) {
            afcloth.SetSubsurfaceColor(Color.white);
            afcloth.SetDiffuseOffset(0.1f);

            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetGravity(1f);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            afcloth.SetFriction(actionConfig.friction);
        }

        protected override IEnumerator Start() {
            yield return WaitForEndOfFrame();
            afcloth.SetSimEnabled(true);
            afcloth.SyncSimValues();
            yield return LerpAlphaToVisible();
        }
    }

    internal class NeckCumDripAction : MouthCumDripAction {

        public NeckCumDripAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { }
    }

    internal class PenisWetAction : AwasFluidAction {

        public PenisWetAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { }

        protected override IEnumerator Start() {
            afcloth.SetThicknessMultiplier(0);
            afcloth.SetSimEnabled(false);
            yield return WaitForEndOfFrame();
            yield return LerpAlphaToVisible();
        }
    }

    internal class PenisWetCumAction : PenisWetAction {

        public PenisWetCumAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { }
    }

    internal class MaleJizzAction : AwasFluidAction {

        AwasFluidCollisionTrigger collisionTrigger = new AwasFluidCollisionTrigger(0.06f);
        AwasFluidPlateCollider fluidPusher = new AwasFluidPlateCollider(0.06f);
        AwasFluidCollisionTrigger tipTrigger = new AwasFluidCollisionTrigger(0.04f);
        // Todo these collision triggers don't need rigidbodies - do something else

        public MaleJizzAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { }

        protected override IEnumerator Stop() {
            tipTrigger.Remove();
            fluidPusher.Remove();
            collisionTrigger.Remove();
            yield return base.Stop();
        }

        protected override IEnumerator Start(FreeControllerV3 pTipController) {
            int tipTriggerCount = 0;

            Vector3 offset = (pTipController.transform.forward * 0.0f) + (pTipController.transform.up * -0.01f);
            collisionTrigger.SetParent(pTipController.transform);
            collisionTrigger.SetTransformWithOffset(pTipController.transform, offset);
            tipTrigger.SetParent(pTipController.transform);
            tipTrigger.SetTransformWithOffset(pTipController.transform, pTipController.transform.forward * 0.03f);
            fluidPusher.SetParent(collisionTrigger.GetTransform());
            fluidPusher.SetTransformWithOffset(collisionTrigger.GetTransform(), Vector3.zero);
            collisionTrigger.SetCollideEnabled(false);

            collisionTrigger.OnTriggerOnce(pTipController.containingAtom, (Collider collider) => {
                tipTrigger.Remove();
                fluidPusher.Remove();
                collisionTrigger.Remove();
                JizzDrip(collider);
            });
            tipTrigger.OnTrigger(pTipController.containingAtom, (Collider collider) => {
                ++tipTriggerCount;
            });

            afcloth.ResetPhysics();
            yield return WaitForFixedUpdateFrames(5);

            afcloth.SetDetatchThreshold(0f);
            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(1f);
            afcloth.SetFriction(actionConfig.friction);
            afcloth.SetGravity(0.3f);
            afcloth.SetSimEnabled(true);
            afcloth.SyncSimValues();

            StartCoroutine(LerpAlphaToVisible(0.5f));
            StartCoroutine(ActivateJizzBlip());

            if (tipTriggerCount > 20) {
                tipTrigger.Remove();
                fluidPusher.Remove();
                collisionTrigger.Remove();

                // TODO not sure what to do here?

                //afcloth.SetDetached(true);
                //SetupClothForDrip();
                //JizzDrip(null);
                //afcloth.SetAlpha(notVisibleAlphaTarget);
                logger.ContextDebug("Tip breakout");
                yield break;
            }

            yield return WaitForFixedUpdateFrames(30);
            Vector3 velocity = pTipController.transform.forward * 1.1f;
            collisionTrigger.SetVelocity(velocity);
            yield return WaitForFixedUpdateFrames(15);
            collisionTrigger.SetGravity(true);
            collisionTrigger.SetCollideEnabled(true);
            fluidPusher.Remove();
            afcloth.SetDetached(true);
            collisionTrigger.SetParent(null);
            yield return new WaitForSecondsRealtime(0.33f);
            SetupClothForDrip();
        }

        private void SetupClothForDrip() {
            afcloth.SetGravity(1f);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
        }

        protected virtual void JizzDrip(Collider targetCollider) {
            afcloth.SetCompressionResistance(actionConfig.dripCompression);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);

            StartCoroutine(LerpCompression(1.5f));
            StartCoroutine(LerpFriction(1.5f));
            StartCoroutine(LerpStiffness(1.5f));
            StartCoroutine(LerpDistanceScale(2f));
        }

        protected IEnumerator ActivateJizzBlip() {
            FreeControllerV3 pMidControl = afcloth.GetContainingAtom().freeControllers.First(rb => rb.name == "penisMidControl");
            FreeControllerV3 pTipControl = afcloth.GetContainingAtom().freeControllers.First(rb => rb.name == "penisTipControl");
            AtomRigidBodyCollisionTrigger tipBlipCollider = new AtomRigidBodyCollisionTrigger(0.05f, pTipControl.GetComponent<Rigidbody>(), "JizzBlip");
            tipBlipCollider.SetTransformWithOffset(pTipControl.transform, Vector3.zero);
            AtomRigidBodyCollisionTrigger midBlipCollider = new AtomRigidBodyCollisionTrigger(0.05f, pMidControl.GetComponent<Rigidbody>(), "JizzBlip");
            midBlipCollider.SetTransformWithOffset(pTipControl.transform, Vector3.zero);
            yield return WaitForFixedUpdateFrames(5);
            tipBlipCollider.Remove();
            midBlipCollider.Remove();
        }
    }

    internal class MaleJizzOnMouthAction : MaleJizzAction {

        AwasFluidCollisionTrigger jizzCollisionTrigger = new AwasFluidCollisionTrigger(0.04f);
        AwasFluidGrabSphere clothGrab = null;

        public MaleJizzOnMouthAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { }

        protected override IEnumerator Stop() {
            jizzCollisionTrigger.Remove();
            clothGrab.Remove();
            yield return base.Stop();
        }

        protected override IEnumerator Start(FreeControllerV3 pTipController) {

            Vector3 offset = pTipController.transform.forward * 0.03f;
            
            yield return AddNewClothGrabSphere(atom => clothGrab = atom);
            jizzCollisionTrigger.SetTransformWithOffset(pTipController.transform, offset);
            clothGrab.SetNewParent(jizzCollisionTrigger.GetTransform());
            clothGrab.GetTransform().rotation = pTipController.transform.rotation;

            jizzCollisionTrigger.OnTriggerOnce(pTipController.containingAtom, (Collider collider) => {
                StartCoroutine(JizzCollide(collider));
            });
            jizzCollisionTrigger.OnTriggerOnce(pTipController.containingAtom, (Collider collider) => {
                jizzCollisionTrigger.Stop();
                jizzCollisionTrigger.Remove();
            });

            clothGrab.Off();
            yield return WaitForEndOfFrame();
            afcloth.SetSimEnabled(true);
            yield return WaitForFixedUpdateFrames(5);

            // mouth jizz setup
            afcloth.SetDetatchThreshold(0f);
            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            afcloth.SetFriction(actionConfig.friction);
            afcloth.SyncSimValues();

            yield return WaitForFixedUpdateFrames(5);
            clothGrab.SetScale(0.4f);
            clothGrab.On();
            StartCoroutine(LerpAlphaToVisible());
            StartCoroutine(ActivateJizzBlip());

            yield return WaitForFixedUpdateFrames(10); 
            Vector3 velocity = pTipController.transform.forward * 1.1f;
            jizzCollisionTrigger.SetVelocity(velocity);
            yield return WaitForFixedUpdateFrames(5);
            jizzCollisionTrigger.SetParent(null);
            float lr = UnityEngine.Random.Range(-0.1f, 0.1f);
            float ud = UnityEngine.Random.Range(-0.1f, 0.1f);
            yield return new WaitForSeconds(0.07f);
            jizzCollisionTrigger.SetVelocity(velocity + (pTipController.transform.right * lr) + (pTipController.transform.up * ud));
        }

        protected IEnumerator JizzCollide(Collider target) {
            float stickyDuration = actionConfig.maxDuration - 3f;

            try {
                JSONStorable lipTriggerStorable = target.GetComponentInParent<Atom>().GetStorableByID("LipTrigger");
                Rigidbody targetRigidBody = lipTriggerStorable.GetComponent<Rigidbody>();
                float lr = UnityEngine.Random.Range(-0.015f, 0.015f);
                float ud = UnityEngine.Random.Range(-0.015f, 0.015f);
                Transform clothGrabTransform = clothGrab.GetTransform();
                clothGrab.SetPosition(targetRigidBody.transform.position + (clothGrabTransform.forward * 0.005f) + (clothGrabTransform.right * lr) + (clothGrabTransform.up * ud));
                clothGrab.SetNewParent(targetRigidBody.transform);
                clothGrab.SetScale(0.2f);
            } catch {
                //clothGrab.Remove();
                //FinishAction();
                //yield break;
                logger.ContextDebug("Collision problem");
            }

            afcloth.SetFriction(actionConfig.dripFriction);
            afcloth.SyncSimValues();
            //yield return new WaitForSeconds(0.5f);

            StartCoroutine(LerpStiffness(1f));
            StartCoroutine(LerpDistanceScale(1f));
            StartCoroutine(LerpCompression(2f));
            StartCoroutine(LerpCollisionPower(0.0f, 0.8f, 1f));
            yield return LerpJointStrength(1f);
            afcloth.SetDetached(true);
            yield return new WaitForSeconds(stickyDuration);
            clothGrab.Remove();
        }
    }

    internal class MaleJizzInMouthAction : MaleJizzAction {

        public MaleJizzInMouthAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { }

        protected override IEnumerator Start(FreeControllerV3 pTipController) {

            yield return WaitForEndOfFrame();
            afcloth.SetSimEnabled(true);
            yield return WaitForFixedUpdateFrames(5);

            // mouth jizz setup
            afcloth.SetDetatchThreshold(0f);
            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            afcloth.SetFriction(actionConfig.friction);
            afcloth.SyncSimValues();

            yield return WaitForFixedUpdateFrames(5);

            StartCoroutine(LerpAlphaToVisible());
            StartCoroutine(ActivateJizzBlip());
            afcloth.SetGravity(0.0f);
            StartCoroutine(LerpForce(pTipController.transform.forward * -35f, Vector3.zero, 0.5f));

            yield return new WaitForSeconds(0.5f);
            afcloth.SetGravity(1f);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            StartCoroutine(LerpStiffness(2f));
            StartCoroutine(LerpDistanceScale(2f));
            StartCoroutine(LerpCompression(2f));
            StartCoroutine(LerpJointStrength(2f));
            yield return LerpFriction(2f);
            afcloth.SetDetached(true);
        }
    }

    internal class MaleStickyJizzAction : MaleJizzAction {

        public MaleStickyJizzAction(AwasFluidCloth awasFluidCloth, AwasActionConfiguration actionConfig) : base(awasFluidCloth, actionConfig) { }

        protected override IEnumerator Start() {
            
            yield return WaitForEndOfFrame();

            afcloth.SetDetatchThreshold(0f);
            afcloth.SetStiffness(actionConfig.stiffness);
            afcloth.SetDistanceScale(actionConfig.size);
            afcloth.SetCompressionResistance(actionConfig.compression);
            afcloth.SetJointStrength(actionConfig.jointStrength);
            afcloth.SetCollisionRadius(actionConfig.dripCollisionRadius);
            afcloth.SetCollisionPower(actionConfig.dripCollisionPower);
            afcloth.SetFriction(actionConfig.friction);
            afcloth.SetSimEnabled(true);
            afcloth.SyncSimValues();

            StartCoroutine(LerpAlphaToVisible());
            StartCoroutine(ActivateJizzBlip());

            yield return LerpJointStrength(1.5f);
            //afcloth.SetDetached(true);
            StartCoroutine(LerpCompression(1.5f));
            StartCoroutine(LerpStiffness(1.5f));
            StartCoroutine(LerpDistanceScale(1.5f));
            StartCoroutine(LerpFriction(1.5f));
        }
    }

}