﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using GPUTools.Cloth.Scripts.Runtime.Physics;
using System;

namespace AwasAutoJuice {
    internal class AwasFluidCloth {
        private DAZClothingItem item;
        private ClothSimControl sim;
        private DAZSkinWrap skin;
        private Material material;
        private DAZSkinWrapControl wrap;
        private DAZSkinWrapMaterialOptions materialOptions;
        private Dictionary<string, float> simFloatDefaults;
        private Dictionary<string, float> wrapFloatDefaults;
        private ClothPhysics phys;
        private DAZClothingItemControl ctrl;
        public string name;
        private bool active; // Prevent this being used until it's back in the pool.

        public AwasFluidCloth(DAZClothingItem clothItem) {
            item = clothItem;
            sim = clothItem.GetComponentInChildren<ClothSimControl>(true);
            skin = clothItem.GetComponentInChildren<DAZSkinWrap>(true);
            wrap = clothItem.GetComponentInChildren<DAZSkinWrapControl>(true);
            phys = clothItem.GetComponentInChildren<ClothPhysics>(true);
            ctrl = clothItem.GetComponentInChildren<DAZClothingItemControl>(true);
            materialOptions = clothItem.GetComponentInChildren<DAZSkinWrapMaterialOptions>(true);

            material = skin.GPUmaterials[0];
            // Remove Awas: from name
            name = sim.transform.name.Remove(0, 5);
            SetDefaults();
            Reset();
            ResetPhysics();
            SetActive(false);
            SetClothActive(false);
            // post load ensures plugin cloth settings are used
            item.PostLoadJSONRestore();

            #region Clothing info
            // *** Clothing Components

            // DAZClothingItem
            // UnityEngine.Transform
            // MeshVR.DAZImport
            // DAZSkinWrapSwitcher
            // DAZSkinWrapControl
            // MeshVR.DAZDynamic
            // MeshVR.PresetManagerControl
            // GPUTools.Cloth.Scripts.ClothSettings
            // ClothSimControl
            // DAZClothingItemControl
            // MeshVR.DAZDynamicDeleter
            // MeshVR.DAZDynamicReloader
            // MeshVR.DAZDynamicItemOpenInCreatorControl
            // MVR.FileManagement.PackageInfo
            // DAZMesh
            // DAZSkinWrap
            // DAZSkinWrapMaterialOptions

            // *** ClothItemSim
            //simEnabled
            //integrateEnabled
            //collisionEnabled
            //collisionRadius
            //drag
            //weight
            //distanceScale
            //stiffness
            //compressionResistance
            //friction
            //staticMultiplier
            //collisionPower
            //gravityMultiplier
            //iterations
            //allowDetach
            //detachThreshold
            //jointStrength
            //force

            // *** Wrap control
            //surfaceOffset
            //additionalThicknessMultiplier
            //wrapToSmoothedVerts
            //smoothIterations
            #endregion
        }

        public bool IsActive() {
            return active;
        }

        public void SetActive(bool active) {
            this.active = active;
        }

        public void SetClothActive(bool active) {
            item.gameObject.SetActive(active);
        }

        public void Delete() {
            // This deletes gameobject but not files. Need to rescan clothes after to get them back in list.
            //ctrl.Delete();
        }

        public void RefreshClothingItems() {
            ctrl.RefreshClothingItems();
        }

        private void SetDefaults() {
            //foreach (string param in sim.GetAllParamAndActionNames()) {
            //    Logger.Debug($"{param}");
            //}
            
            string[] simFloats = { 
                "collisionRadius", "drag", "weight", "distanceScale", "stiffness", "compressionResistance", "friction", "staticMultiplier",
                "collisionPower", "gravityMultiplier", "iterations", "allowDetach", "detachThreshold", "jointStrength" 
            };
            simFloatDefaults = new Dictionary<string, float>();
            foreach (string floatName in simFloats) { 
                simFloatDefaults.Add(floatName, sim.GetFloatParamValue(floatName));
            }
            string[] wrapFloats = {
                "surfaceOffset", "additionalThicknessMultiplier"
            };
            wrapFloatDefaults = new Dictionary<string, float>();
            foreach (string floatName in wrapFloats) {
                wrapFloatDefaults.Add(floatName, wrap.GetFloatParamValue(floatName));
            }
        }

        public void RestoreDefaults() {
            foreach (KeyValuePair<string, float> simDefault in simFloatDefaults) {
                sim.SetFloatParamValue(simDefault.Key, simDefault.Value);
            }
            foreach (KeyValuePair<string, float> wrapDefault in wrapFloatDefaults) {
                sim.SetFloatParamValue(wrapDefault.Key, wrapDefault.Value);
            }
        }

        public Atom GetContainingAtom() {
            return item.containingAtom;
        }

        public void SetSkinScale(float scale) {
            skin.transform.localScale = new Vector3(scale, scale, scale);

        }

        public void SetAlpha(float alpha) {
            for (int i = 0; i < skin.GPUmaterials.Length; ++i) {
                if (skin.GPUmaterials[i].HasProperty("_AlphaAdjust")) {
                    skin.GPUmaterials[i].SetFloat("_AlphaAdjust", alpha);
                }
            }
        }

        public void SetColor(Color color) {
            for (int i = 0; i < skin.GPUmaterials.Length; ++i) {
                skin.GPUmaterials[i].color = color;
            }
        }

        public Color GetColor() {
            return material.color;
        }

        public void SetIntegration(bool enabled) {
            sim.SetBoolParamValue("integrateEnabled", enabled);
        }

        public void SetSubsurfaceColor(Color color) {
            materialOptions.SetColorParamValue("Subsurface Color", HSVColorPicker.RGBToHSV(color.r, color.g, color.b));
        }

        public void SetDiffuseOffset(float offset) {
            materialOptions.SetFloatParamValue("Diffuse Texture Offset", offset);
        }

        public float GetAlpha() {
            for (int i = 0; i < skin.GPUmaterials.Length; ++i) {
                if (skin.GPUmaterials[i].HasProperty("_AlphaAdjust")) {
                    return skin.GPUmaterials[i].GetFloat("_AlphaAdjust");
                }
            }
            return 0f;
        }

        public void SetSimEnabled(bool isEnabled) {
            sim.SetSimEnabled(isEnabled);
        }

        public void SetDetached(bool detached) {
            sim.SetBoolParamValue("allowDetach", detached);
        }

        public void SetDistanceScale(float scale) {
            sim.SetFloatParamValue("distanceScale", scale);
        }

        public void SetDetatchThreshold(float threshold) {
            sim.SetFloatParamValue("detachThreshold", threshold);
        }

        public void SetForce(Vector3 force) {
            sim.SetVector3ParamValue("force", force);
        }

        public void SetSurfaceOffset(float offset) {
            wrap.SetFloatParamValue("surfaceOffset", offset);
        }

        public void SetThicknessMultiplier(float thickness) {
            wrap.SetFloatParamValue("additionalThicknessMultiplier", thickness);
        }

        public void SetWeight(float weight) {
            sim.SetFloatParamValue("weight", weight);
        }

        public void SetDrag(float drag) {
            sim.SetFloatParamValue("drag", drag);
        }

        public void SetGravity(float gravity) {
            sim.SetFloatParamValue("gravityMultiplier", gravity);
        }

        public void SetJointStrength(float strength) {
            sim.SetFloatParamValue("jointStrength", strength);
        }

        public void SetStiffness(float stiffness) {
            sim.SetFloatParamValue("stiffness", stiffness);
        }

        public void SetFriction(float friction) {
            sim.SetFloatParamValue("friction", friction);
        }

        public void SetCollisionPower(float collision) {
            sim.SetFloatParamValue("collisionPower", collision);
        }

        public void SetCollisionRadius(float radius) {
            sim.SetFloatParamValue("collisionRadius", radius);
        }

        public void SetCollisionEnabled(bool enabled) {
            sim.SetBoolParamValue("collisionEnabled", enabled);
        }

        public void SetCompressionResistance(float resistance) {
            sim.SetFloatParamValue("compressionResistance", resistance);
        }

        public void SyncSimValues() {
            item.SyncClothingAdjustments();
        }

        public void ResetPhysics() {
            item.ResetPhysics();
        }

        public void Reset() {
            SetAlpha(-1f);
            SetForce(Vector3.zero);
            SetDetached(false);
        }
    }
}
