﻿using GPUTools.Physics.Scripts.Behaviours;
using System;
using UnityEngine;

namespace AwasAutoJuice {
    internal class AwasFluidPlateCollider {
        private readonly GameObject colliderSphere;

        public AwasFluidPlateCollider(float localScale) {
            colliderSphere = GameObject.CreatePrimitive(PrimitiveType.Sphere);
            colliderSphere.transform.localScale = new Vector3(localScale, localScale, localScale * 0.01f);
            colliderSphere.GetComponent<SphereCollider>().isTrigger = true;
            colliderSphere.GetComponent<SphereCollider>().radius = 0.75f;

            try {
                GpuSphereCollider gpc = colliderSphere.gameObject.AddComponent<GpuSphereCollider>();
                gpc.radius = 0.5f;
                gpc.friction = 10f;
            } catch (Exception e) {
                SuperController.LogError(e.Message);
            }

            colliderSphere.GetComponent<Renderer>().material = new Material(Shader.Find("Battlehub/RTGizmos/Handles")) { color = new Color(1f, 0.1f, 0.1f, 0.3f) };
            colliderSphere.GetComponent<Renderer>().enabled = false;
        }

        public AwasFluidPlateCollider(Transform parent, Vector3 positionOffset, float localScale = 0.04f) : this(localScale) {
            colliderSphere.transform.SetParent(parent);
            colliderSphere.transform.SetPositionAndRotation(parent.position + positionOffset, parent.rotation);
        }

        public void SetScale(float localScale) {
            colliderSphere.transform.localScale = new Vector3(localScale, localScale, localScale);
        }

        public void SetParent(Transform newParent) {
            colliderSphere.transform.SetParent(newParent);
        }

        public Transform GetTransform() {
            return colliderSphere.transform;
        }

        public void SetTransformWithOffset(Transform source, Vector3 positionOffset) {
            colliderSphere.transform.SetPositionAndRotation(source.position + positionOffset, source.rotation);
        }

        public void Remove() {
            UnityEngine.Object.Destroy(colliderSphere);
        }

        public void Debug() {
            colliderSphere.GetComponent<Renderer>().enabled = true;
        }
    }
}
