﻿using UnityEngine;

namespace AwasAutoJuice {
    internal class AwasFluidSphere {
        private Atom sphere;
        private Transform originalTransformParent;
        private Transform originalControlTransformParent;

        public AwasFluidSphere(Atom sphereAtom) {
            sphere = sphereAtom;
            originalTransformParent = sphere.transform.parent;
            originalControlTransformParent = sphere.mainController.transform.parent;
        }

        public void SetScale(float scale) {
            sphere.GetStorableByID("scale").SetFloatParamValue("scale", scale);
        }

        public void On() {
            sphere.SetOn(true);
        }

        public void Off() {
            sphere.SetOn(false);
        }

        public void SetCollisionOnlyMode() {
            sphere.GetComponent<Rigidbody>().isKinematic = true;
            sphere.GetComponent<Rigidbody>().detectCollisions = false;
        }

        public void SetTransformWithOffset(Transform transform, Vector3 offset) {
            sphere.transform.SetPositionAndRotation(transform.position + offset, transform.rotation);
            sphere.mainController.transform.SetPositionAndRotation(transform.position + offset, transform.rotation);
        }

        public void SetPosition(Vector3 newPosition) {
            sphere.transform.position = newPosition;
            sphere.mainController.transform.position = newPosition;
        }

        public void SetNewParent(Transform newParent) {
            sphere.transform.SetParent(newParent);
            sphere.mainController.transform.SetParent(newParent);
        }

        public void Remove() {
            RestoreOriginalParent();
            SuperController.singleton.RemoveAtom(sphere);
        }

        private void RestoreOriginalParent() {
            sphere.transform.SetParent(originalTransformParent);
            sphere.mainController.transform.SetParent(originalControlTransformParent);
        }

        public Transform GetTransform() {
            return sphere.transform;
        }
    }
}
